# FILE: accountGui.tcl
#

em_class net_accountGui {
    const public variable winpath
    const public variable outer_frame_options = "-bd 4 -relief ridge"
    const public variable account
    private variable accountlabel
    private variable programs
    private variable threads
    private variable plabel
    private variable tlabel
    private variable qhandler
    private variable thandler
    private variable threadbuttons = {}
    private variable myProgramButtons = {}
    private variable myEventHandlers = {}
    constructor {basewinpath args} {
	# Don't add '.' suffix if there already is one.
	# (This happens chiefly when basewinpath is exactly '.')
	if {![string match {.} \
		[string index $basewinpath \
		[expr [string length $basewinpath]-1]]]} {
	    append basewinpath {.}
	}
	eval $this configure -winpath ${basewinpath}w$this $args

	# Pack entire subwidget into a subframe.  This makes
	# housekeeping easier, since the only bindings on
	# $winpath will be ones added from inside the class,
	# and also all subwindows can be destroyed by simply
	# destroying $winpath.  (The instance destructor should
	# not "destroy" $frame, because $frame is owned external
	# to this class.
	eval frame $winpath $outer_frame_options
        set accountname [$account cget -accountname]
        set accountlabel [label $winpath.label -text $accountname \
                -relief groove]
        pack $accountlabel -side top -fill x
        set programs [frame $winpath.programs -relief groove]
        set threads [frame $winpath.threads -relief groove]
        set plabel [label $programs.label -text Programs -relief flat]
        set tlabel [label $threads.label -text Threads -relief flat]
        pack $plabel -side top -in $programs -fill x
        pack $tlabel -side top -in $threads -fill x
        pack $programs -side left -fill both  -expand 1
        pack $threads -side left -fill both -expand 1
        $this DisplayThreadButtons
        lappend myEventHandlers [em_eventHandler new net_thread delete \
                [list $this DisplayThreadButtons]]
        lappend myEventHandlers [em_eventHandler new net_thread Ready \
                [list $this DisplayThreadButtons]]
        set qid [$account Send programs]
        set qhandler [em_eventHandler new $account Reply$qid \
                [list $this ProgramsReply]]
        set thandler [em_eventHandler new $account Timeout$qid \
                [list $this ProgramsTimeout]]
    }
    method DisplayThreadButtons {} {
        foreach w $threadbuttons {
            destroy $w
        }
        set threadbuttons {}
        foreach thread [net_thread instances] {
            if {[string match [$account cget -accountname] \
                    [$thread cget -accountname]]} {
                if {[$thread Ready]} {
                    if {[$thread HasGui]} {
                        set btn [checkbutton $threads.w$thread \
                              -text [$thread cget -alias]<[$thread cget -pid]> \
                              -variable ${thread}_BtnVar \
                              -command [list $thread ToggleGui]]
                    } else {
                        set btn [label $threads.w$thread \
                              -text [$thread cget -alias]<[$thread cget -pid]>]
                    }
                    lappend threadbuttons $btn
                    pack $btn -side top -in $threads -fill x
                }
            }
        }
    }
    method ProgramsReply {} {
        $qhandler delete
        $thandler delete
        unset qhandler
        unset thandler
        set result [$account Get]
        if {![lindex $result 0]} {
            foreach pgm [lrange $result 1 end] {
                set btn [button $programs.$pgm -text $pgm]
                $btn configure -command [list $this LaunchProgram $pgm $btn]
                lappend myProgramButtons $btn
                pack $btn -side top -fill x 
            }
        }
    }
    method LaunchProgram { pgm btn } {
        $btn configure -state disabled
        set qid [$account Send launch $pgm]
        set qhandler [em_eventHandler new $account Reply$qid \
                [list $this LaunchReply $btn]]
        set thandler [em_eventHandler new $account Timeout$qid \
                [list $this LaunchTimeout $btn]]
    }
    method LaunchReply { btn } {
        if {[winfo exists $btn]} {
            $btn configure -state normal
        }
    }
    method LaunchTimeout { btn } {
        if {[winfo exists $btn]} {
            $btn configure -state normal
        }
    }
    method ProgramsTimeout {} {
        $qhandler delete
        $thandler delete
        unset qhandler
        unset thandler
        # Need to report error somehow!
    }
    method WinDestroy { w } { winpath } {
	if {[string compare $winpath $w]!=0} { return } ;# Child destroy event
	$this delete
    }
    destructor {
        foreach handler $myEventHandlers {
            $handler delete
        }
        if {[info exists qhandler]} {
            $qhandler delete
        }
        if {[info exists thandler]} {
            $thandler delete
        }
	# Delete everything else
	if {[info exists winpath] && [winfo exists $winpath]} {
            # Remove <Destroy> binding
	    bind $winpath <Destroy> {}
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}
