# FILE: hostGui.tcl
#

em_class net_hostGui {
    const public variable winpath
    const public variable outer_frame_options = "-bd 4 -relief ridge"
    public variable host
    private variable acctbuttons = {}
    private variable ctrlbar
    private variable panel
    private variable myEventHandlers = {}
    constructor {basewinpath args} {
	# Don't add '.' suffix if there already is one.
	# (This happens chiefly when basewinpath is exactly '.')
	if {![string match {.} \
		[string index $basewinpath \
		[expr [string length $basewinpath]-1]]]} {
	    append basewinpath {.}
	}
	eval $this configure -winpath ${basewinpath}w$this $args

	# Pack entire subwidget into a subframe.  This makes
	# housekeeping easier, since the only bindings on
	# $winpath will be ones added from inside the class,
	# and also all subwindows can be destroyed by simply
	# destroying $winpath.  (The instance destructor should
	# not "destroy" $frame, because $frame is owned external
	# to this class.
	eval frame $winpath $outer_frame_options
        set ctrlbar [frame $winpath.ctrl -relief groove]
        set panel [frame $winpath.panel -relief groove]
        set hostname [$host cget -hostname]
        set hostlabel [label $ctrlbar.label -text $hostname:]
        pack $hostlabel -side left -in $ctrlbar -fill x -expand 0
        $this DisplayAccountButtons
        lappend myEventHandlers [em_eventHandler new net_account delete \
                [list $this DisplayAccountButtons]]
        lappend myEventHandlers [em_eventHandler new net_account Ready \
                [list $this DisplayAccountButtons]]
        pack $ctrlbar -side top -fill x -expand 0
        pack $panel -side top -fill both -expand 1
    }
    method DisplayAccountButtons {} {
        foreach w $acctbuttons {
            destroy $w
        }
        set acctbuttons {}
        foreach acct [net_account instances] {
            if {[string match [$host cget -hostname] [$acct cget -hostname]]} {
                if {[$acct Ready]} {
                    set btn [checkbutton $ctrlbar.w$acct \
                            -text [$acct cget -accountname] \
                            -variable ${acct}_BtnVar \
                            -command [list $acct ToggleGui $panel]]
                    lappend acctbuttons $btn
                    pack $btn -side left -in $ctrlbar -expand 0
                }
            }
        }
    }
    method WinDestroy { w } { winpath } {
	if {[string compare $winpath $w]!=0} { return } ;# Child destroy event
	$this delete
    }
    destructor {
        foreach handler $myEventHandlers {
            $handler delete
        }
        foreach w $acctbuttons {
            set varname [$w cget -variable]
            global $varname
            if {[set $varname]} {
                $w invoke
            }
        }
	# Delete everything else
	if {[info exists winpath] && [winfo exists $winpath]} {
            # Remove <Destroy> binding
	    bind $winpath <Destroy> {}
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}
