# FILE: threadGui.tcl
#

em_class net_threadGui {
    const public variable winpath
    const public variable outer_frame_options = "-bd 4 -relief ridge"
    public variable thread
    private variable inputs
    private variable ctrl 
    private variable cmdbtns
    private variable outputs
    private variable cmdhandler 
    private variable ophandler 
    private variable iphandler
    private variable evhandler 
    private variable ichandler
    private variable dataselection
    private variable indataselection
    private variable threadselection
    private variable ithreadselection
    private variable dt
    private variable idt
    private variable schedw
    private variable tdhandler
    private variable trhandler
    private variable itdhandler
    private variable itrhandler
    private variable opdb
    private variable ipdb
    private variable evdb
    private variable myEventHandlers = {}
    constructor {args} {
	eval $this configure -winpath .w$this $args
        toplevel $winpath
        wm group $winpath
        wm title $winpath [$thread cget -alias]<[$thread cget -pid]>
	after idle "OMF_SetIcon $winpath"

        # Input control panel
        set inputs [frame $winpath.inputs -bd 4 -relief ridge]
        global _cb$thread
        set _cb${thread}(in) 1
        pack [checkbutton $inputs.label -text "Inputs" \
                -variable _cb${thread}(in) \
                -command [list $this DisplayInputs] -relief groove] \
                -side top -in $inputs -fill x
        pack $inputs -side top -fill both -expand 1

        # Runtime control panel with exit button
        set ctrl [frame $winpath.ctrl -bd 4 -relief ridge]
        pack [label $ctrl.label -text "Runtime Control" -relief groove] \
                -side top -in $ctrl -fill x
        pack $ctrl -side top -fill both -expand 1
        pack [button $ctrl.exit -text Exit \
                -command [list $thread Send delete]] \
                -in $ctrl -side right -fill both

        # Output schedule control panel
        set outputs [frame $winpath.outputs -bd 4 -relief ridge]
        global _cb$thread
        set _cb${thread}(so) 0
        pack [checkbutton $outputs.label -text "Scheduled Outputs" \
                -variable _cb${thread}(so) \
                -command [list $this DisplayOutputs] -relief groove] \
                -side top -in $outputs -fill x
        pack $outputs -side top -fill both -expand 1
        bind $winpath <Destroy> "$this WinDestroy %W"
        $this ResetInterface
    }
    method ResetInterface {} {
        set cmdhandler [em_eventHandler new $thread \
                Reply[$thread Send Commands] [list $this ReceiveCommands]]
        set ophandler [em_eventHandler new $thread \
                Reply[$thread Send Outputs] [list $this ReceiveOutputs]]
        set iphandler [em_eventHandler new $thread \
                Reply[$thread Send Inputs] [list $this ReceiveInputs]]
        set evhandler [em_eventHandler new $thread \
                Reply[$thread Send Events] [list $this ReceiveEvents]]
        set ichandler [em_eventHandler new $thread Readable \
                [list $this DetectInterfaceChange]]
        catch {$opdb delete}
        catch {$ipdb delete}
        catch {$evdb delete}
        set opdb [em_array new]
        set ipdb [em_array new]
        set evdb [em_array new]
    }
    method DetectInterfaceChange {} {
        set message [$thread Get]
        if {[string match InterfaceChange $message]} {
            $ichandler delete
            unset ichandler
            $this ResetInterface
        }
    }
    method ReceiveCommands {} {
        $cmdhandler delete
        unset cmdhandler
        set cmds [$thread Get]
        if {![lindex $cmds 0]} { ;# Check for error
	set min_label_width 75
        if {[info exists cmdbtns] && [winfo exists $cmdbtns]} {
            destroy $cmdbtns
        }
        set cmdbtns [frame $ctrl.cmdbtns]
        foreach {l group} [lindex $cmds 1] {
            frame $cmdbtns.w$l
	    frame $cmdbtns.w$l.labelframe
	    frame $cmdbtns.w$l.labelspacer -width $min_label_width
	    label $cmdbtns.w$l.label -text "${l}:"
	    pack $cmdbtns.w$l.labelspacer -in $cmdbtns.w$l.labelframe -side top
            pack $cmdbtns.w$l.label -in $cmdbtns.w$l.labelframe -side right \
		    -anchor e
            pack $cmdbtns.w$l.labelframe -in $cmdbtns.w$l -side left -fill both
            foreach c $group {
                set text [lindex $c 0]
                # This might impose restriction that labels contain no spaces
                pack [button $cmdbtns.w$l.w$text -text $text \
                        -command [concat $thread Send [lrange $c 1 end]]] \
                        -in $cmdbtns.w$l -side left -fill both
            }
            pack $cmdbtns.w$l -in $cmdbtns -side top -fill x
        }
        pack $cmdbtns -in $ctrl -side top -fill x
        }
    }
    method ReceiveOutputs {} {
        $ophandler delete
        unset ophandler
        set op [$thread Get]
        if {![lindex $op 0]} { ;# Check for error
            foreach o [$opdb names] {
                $opdb unset $o
            }
            foreach {o desc} [lindex $op 1] {
                $opdb set [list $o $desc]
            }
            $this DisplayInteractiveOutputButtons
            $this DisplayOutputs
        }
    }
    method ReceiveInputs {} {
        $iphandler delete
        unset iphandler
        set ip [$thread Get]
        if {![lindex $ip 0]} { ;# Check for error
            foreach o [$ipdb names] {
                $ipdb unset $o
            }
            foreach {o desc} [lindex $ip 1] {
                $ipdb set [list $o $desc]
            }
            $this DisplayInputs
        }
    }
    method DisplayInteractiveOutputButtons {} {
        if {[winfo exists $ctrl.io]} {
            destroy $ctrl.io
        }
        frame $ctrl.io
	frame $ctrl.io.labelframe
	frame $ctrl.io.labelspacer -width 75
	label $ctrl.io.label -text "Interactive\nOutputs:" -justify right
	pack $ctrl.io.labelspacer -in $ctrl.io.labelframe -side top
	pack $ctrl.io.label -in $ctrl.io.labelframe -side right -anchor e
        pack $ctrl.io.labelframe -in $ctrl.io -side left -fill both
        foreach o [$opdb names] {
            pack [button $ctrl.io.w$o -text $o \
                -command [list $this InteractiveOutput $o]] \
                -in $ctrl.io -side left -fill both
        }
        pack $ctrl.io -side bottom -fill both -expand 1
    }
    method DisplayInputs {} {
        if {[winfo exists $inputs.data]} {
            destroy $inputs.data
        }
        global _cb$thread
        if {![set _cb${thread}(in)]} {
            $this DisplayInputThreads ""
            return
        }
        set data [frame $inputs.data -bd 4 -relief ridge]
        pack [label $data.label -text "Inputs" -relief groove] \
                -side top -in $data -fill x
        foreach o [$ipdb names] {
            set desc [$ipdb lookup $o]
            pack [radiobutton $data.w$o -text $o -value $o \
                    -variable [$this globalname indataselection] \
                    -anchor w \
                    -command [list $this DisplayInputThreads [lindex $desc 0]]] \
                    -side top -in $data -fill x
        }
        pack $data -side left -fill y
        if {[$ipdb size]} {
            set indataselection [lindex [$ipdb names] 0]
        } else {
            catch {unset indataselection}
        }
        $this DisplayInputThreads ""
    }
    method DisplayOutputs {} {
        if {[winfo exists $outputs.data]} {
            destroy $outputs.data
        }
        global _cb$thread
        if {![set _cb${thread}(so)]} {
            $this DisplayThreads ""
            return
        }
        set data [frame $outputs.data -bd 4 -relief ridge]
        pack [label $data.label -text "Outputs" -relief groove] \
                -side top -in $data -fill x
        foreach o [$opdb names] {
            set desc [$opdb lookup $o]
            pack [radiobutton $data.w$o -text $o -value $o \
                    -variable [$this globalname dataselection] \
                    -anchor w \
                    -command [list $this DisplayThreads [lindex $desc 0]]] \
                    -side top -in $data -fill x
        }
        pack $data -side left -fill y
        if {[$opdb size]} {
            set dataselection [lindex [$opdb names] 0]
        }
        $this DisplayThreads ""
    }
    method ReceiveEvents {} {
        $evhandler delete
        unset evhandler
        set ev [$thread Get]
        if {![lindex $ev 0]} { ;# Check for error
        foreach {e val} [lindex $ev 1] {
            $evdb set [list $e $val]
        }
        }
    }
    method DisplayThreads { type } {
        if {[info exists dt]} {
            destroy $dt
            unset dt
        }
        if {[info exists schedw]} {
            destroy $schedw
            unset schedw
        }
        if {[info exists tdhandler]} {
            $tdhandler delete
            unset tdhandler
        }
        if {[info exists trhandler]} {
            $trhandler delete
            unset trhandler
        }
        catch {unset threadselection}
        global _cb$thread
        if {![set _cb${thread}(so)]} {
            $this DisplaySchedule
            return
        }
        if {![string length $type] && [info exists dataselection] \
                && [string length $dataselection]} {
            set desc [$opdb lookup $dataselection]
            set type [lindex $desc 0]
        }
        set dt [frame $outputs.dt -bd 4 -relief ridge]
        pack [label $dt.label -text "Destination Threads" -relief groove] \
                -side top -in $dt -fill x
        foreach t [net_thread instances] {
            if {[$t Ready]} {
                if {[string match $type [$t Protocol]]} {
                    set btn [radiobutton $dt.w$t \
                            -text [$t cget -alias]<[$t cget -pid]> \
                            -value $t -anchor w \
                            -command [list $this DisplaySchedule] \
                            -variable [$this globalname threadselection]]
                    pack $btn -side top -in $dt -fill x
                }
            }
        }
        pack $dt -side left -fill both -expand 1
        $this DisplaySchedule
        # Set up for redraw when thread dies or new thread becomes ready
        set tdhandler [em_eventHandler new net_thread delete \
                [list $this DisplayThreads $type]]
        set trhandler [em_eventHandler new net_thread Ready \
                [list $this DisplayThreads $type]]
    }
    method DisplayInputThreads { type } {
        if {[info exists idt]} {
            destroy $idt
            unset idt
        }
        if {[info exists itdhandler]} {
            $itdhandler delete
            unset itdhandler
        }
        if {[info exists itrhandler]} {
            $itrhandler delete
            unset itrhandler
        }
        catch {unset ithreadselection}
        set idt [frame $inputs.dt -bd 4 -relief ridge]
        pack [label $idt.label -text "Source Threads" -relief groove] \
                -side top -in $idt -fill x
        global _cb$thread
        if {![set _cb${thread}(in)]} {
            return
        }
        if {![string length $type] && [info exists indataselection] \
                && [string length $indataselection]} {
            set desc [$ipdb lookup $indataselection]
            set type [lindex $desc 0]
[em_extension ApplicationLog] Append "type = $type"
        }
        foreach t [net_thread instances] {
[em_extension ApplicationLog] Append "Thread $t: [$t cget -pid]"
            if {[$t Ready]} {
[em_extension ApplicationLog] Append "Ready Thread $t: [$t cget -pid]"
               if {[string match $type [$t Protocol]]} {
[em_extension ApplicationLog] Append "Match Thread $t: [$t cget -pid]"
                   set btn [radiobutton $idt.w$t \
                            -text [$t cget -alias]<[$t cget -pid]> \
                            -value $t -anchor w \
                            -command [list $this UpdateInput] \
                            -variable [$this globalname ithreadselection]]
                    pack $btn -side top -in $idt -fill x
                } else {
[em_extension ApplicationLog] Append "UnMatch Thread $t: [$t cget -pid],$type != [$t Protocol]"
                }
            } 
        }
        pack $idt -side left -fill both -expand 1
        # Set up for redraw when thread dies or new thread becomes ready
        set itdhandler [em_eventHandler new net_thread delete \
                [list $this DisplayInputThreads $type]]
        set itrhandler [em_eventHandler new net_thread Ready \
                [list $this DisplayInputThreads $type]]
    }
    method DisplaySchedule {} {
        if {[info exists schedw]} {
            destroy $schedw
            unset schedw
        }
        global _cb$thread
        foreach iii [array names _cb$thread *,*,*] {
            regexp {([^,]*),([^,]*),([^,]*)} $iii match d t e
            if {[catch {$t Ready} ready] || !$ready} {
                $thread Send UnsetHandler $e $d \
                        [$t cget -hostname] \
                        [$t cget -accountname] \
                        [$t cget -pid]
                unset _cb${thread}($iii)
            }
        }
        if {![set _cb${thread}(so)]} {
            return
        }
        set schedw [frame $outputs.sched -bd 4 -relief ridge]
        pack [label $schedw.label -text "Schedule" -relief groove] \
                -side top -in $schedw -fill x
        if {[info exists dataselection] && [string length $dataselection]} {
        set opinfo [$opdb lookup $dataselection]
        if {[info exists threadselection] && [string length $threadselection]} {
        foreach ev [lindex $opinfo 1] {
            set mf [frame $schedw.mf$ev]
            set btn [checkbutton $mf.w$ev -text $ev -anchor w \
                    -variable _cb${thread}($dataselection,$threadselection,$ev) \
                    -command [list $this UpdateSchedule $ev]]
            pack $btn -side left -in $mf -fill x
            global _cnt$thread
            if {![info exists \
                    _cnt${thread}($dataselection,$threadselection,$ev)]} {
                set _cnt${thread}($dataselection,$threadselection,$ev) 1
            }
            set evinfo [$evdb lookup $ev]
            if {[lindex $evinfo 0]} {
                set cnt [entry $mf.cnt \
                -textvariable _cnt${thread}($dataselection,$threadselection,$ev)]
                pack $cnt -side right -fill x
                bind $cnt <Return> [list $this UpdateSchedule $ev]
                set lbl [label $mf.lbl -text every]
                pack $lbl -side right -fill x
            }
            pack $mf -side top -fill x
        }
        set mf [frame $schedw.interactive]
        set btn [checkbutton $mf.btn -text Interactive -anchor w \
            -variable _cb${thread}($dataselection,$threadselection,interactive)]
        pack $btn -side left -in $mf -fill x
        pack $mf -side top -fill x
        }
        }
        pack $schedw -side right -fill both -expand 1
    }
    method UpdateSchedule { event } {
        global _cb$thread
        global _cnt$thread
        if {[set _cb${thread}($dataselection,$threadselection,$event)]} {
            $thread Send SetHandler $event $dataselection \
                    [$threadselection cget -hostname] \
                    [$threadselection cget -accountname] \
                    [$threadselection cget -pid] \
                    [set _cnt${thread}($dataselection,$threadselection,$event)]
        } else {
            $thread Send UnsetHandler $event $dataselection \
                    [$threadselection cget -hostname] \
                    [$threadselection cget -accountname] \
                    [$threadselection cget -pid]
        }
    }
    method UpdateInput {} {
        $thread Send SetInput $indataselection \
                [$ithreadselection cget -hostname] \
                [$ithreadselection cget -accountname] \
                [$ithreadselection cget -pid]
    }
    method InteractiveOutput { data } {
        global _cb$thread
        foreach iii [array names _cb$thread $data,*,interactive] {
            regexp $data,(\[^,]*),interactive $iii match t
            $thread Send RemoteRelayOutput $data [$t cget -hostname] \
                    [$t cget -accountname] [$t cget -pid]
        }
    }
    method WinDestroy { w } { winpath } {
	if {[string compare $winpath $w]!=0} { return } ;# Child destroy event
	$this delete
    }
    destructor {
        if {[info exists tdhandler]} {
            $tdhandler delete
        }
        if {[info exists trhandler]} {
            $trhandler delete
        }
        if {[info exists itdhandler]} {
            $itdhandler delete
        }
        if {[info exists itrhandler]} {
            $itrhandler delete
        }
        if {[info exists iphandler]} {
            $iphandler delete
        }
        if {[info exists ophandler]} {
            $ophandler delete
        }
        if {[info exists cmdhandler]} {
            $cmdhandler delete
        }
        if {[info exists evhandler]} {
            $evhandler delete
        }
        if {[info exists ichandler]} {
            $ichandler delete
        }
        foreach handler $myEventHandlers {
            $handler delete
        }
	# Delete everything else
	if {[info exists winpath] && [winfo exists $winpath]} {
            # Remove <Destroy> binding
	    bind $winpath <Destroy> {}
	    # Destroy instance frame, all children, and bindings
	    destroy $winpath
	}
    }
}
